## Replication files for 
## "Cluster Robust Covariance Matrix Estimation in Panel Quantile Regression with Individual Fixed Effects"
## by Jungmo Yoon and Antonio Galvao
## This is a collection of R functions for CCM estimator

# 1. Funcnctions that estimate the cluster robust variance-covariance matrix

ccm.se = function(x,e0,e1,uhat,tau,nt,ng,mt,se,bias,ker){
	# calculate CCM estimator
	# 'x' includes individual fixed effects
	# se=1 for the cluster robust standard error, 0 for non-robust standard error
	# bias=1 then estimate and correct the bias in autocovariance, 0 then do not correct the bias term
	# ker = 1,2,3 for truncated, Bartlett, and QS kernel, respectively
	
	fxxinv = e1*(nt*ng)
	if(se==0){D0 = e0/(nt*ng)}
	if(se==1){
	# beginning of the autocovariance estimation
	dd = step.one(nt,ng,uhat,tau,mt[1])	# bias estimation
	q = ncol(x)
	xx1 = array(0,c(q,q,(nt-1)))
	hx1 = array(0,c(q,q,(nt-1)))
	for(s in 0:(nt-2)){
		xx = 0; hx = 0
		for(i in 1:ng){
			sel = ((i-1)*nt+1):(i*nt)
			xh  = x[sel,]
			xh1 = xh[1:(nt-s),]
			xh2 = xh[(1+s):nt,]
			uh  = tau - (uhat[sel] < 0)
			xu1 = xh1*uh[1:(nt-s)]
			xu2 = xh2*uh[(1+s):nt]
			xx = xx + crossprod(xh1,xh2)/(nt-s)
			hx = hx + crossprod(xu1,xu2)/(nt-s)
		}
	xx1[,,(s+1)] = xx/ng
	hx1[,,(s+1)] = hx/ng
	}
	if(bias==0){res <- step.long(nt,ng,hx1,mt[2],ker)}
	if(bias==1){
		hx2 <- bias.cor(hx1,xx1,dd$v.hat,nt,opt=1)
		res <- step.long(nt,ng,hx2,mt[2],ker)
	}
	D0  <- res$J.r
	Ds0 <- res$J.n	# variance matrix estimator assumeing no serial correction
	}
	cov = (fxxinv %*% D0 %*% fxxinv)/(nt*ng)
	if(se==0){covs = NULL}
	if(se==1){covs = (fxxinv %*% Ds0 %*% fxxinv)/(nt*ng)} # assumes no serial correction
	return(list(cov = cov, covs = covs, D0 = D0, D1.inv = fxxinv, uhat = uhat))
}

ccm.se.s = function(x,e0,e1,uhat,tau,nt,ng,mt,se,bias,ker){
	# a special case of ccm.se()
	# calculates the simplified CCM estimator
	# valid for the homoskedastic errors

	fxxinv = e1*(nt*ng)
	if(se==0){D0 = e0/(nt*ng)}
	if(se==1){
	# beginning of the autocovariance estimation
	dd = step.one(nt,ng,uhat,tau,mt[1])	# bias estimation
	q = ncol(x)
	xx1 = array(0,c(q,q,(nt-1)))
	for(s in 0:(nt-2)){
		xx = 0; hx = 0
		for(i in 1:ng){
			sel = ((i-1)*nt+1):(i*nt)
			xh  = x[sel,]
			xh1 = xh[1:(nt-s),]
			xh2 = xh[(1+s):nt,]
			xx = xx + crossprod(xh1,xh2)/(nt-s)
		}
	xx1[,,(s+1)] = xx/ng
	}
	if(bias==0){rr = dd$rho.hat.nb}
	if(bias==1){rr = dd$rho.hat}
	Jm = rr[1]*xx1[,,1]
	if(ker==1){ # truncated kernel
		for(s in 1:mt[2]){Jm = Jm + 2*((nt-s)/nt)*rr[s+1]*xx1[,,(s+1)]}
	}
	if(ker==2){ # Bartlett kernel
		for(s in 1:mt[2]){Jm = Jm + 2*(1-abs(s/mt[2]))*((nt-s)/nt)*rr[s+1]*xx1[,,(s+1)]}
	}
	if(ker==3){ # QS kernel
		for(s in 1:(nt-2)){
			z = s/mt[2]
			Jm = Jm + 2*(25/(12*(pi^2)*(z^2)))*(sin(6*pi*z/5)/(6*pi*z/5)-cos(6*pi*z/5))*((nt-s)/nt)*rr[s+1]*xx1[,,(s+1)]
		}
	}
	D0 <- Jm
	}
	cov = (fxxinv %*% D0 %*% fxxinv)/(nt*ng)
	return(list(cov = cov, D0 = D0, D1.inv = fxxinv, uhat = uhat))
}

# 2. Functions that estimate bias terms in auto-covariance matrix

step.one <- function(nt,ng,uhat,tau,mt){
	# estimate the bias of auto-covariances and long-run variance
	# for simplicity, this function uses the truncated kernel 

	# 1. auto-covariances estimation
	rho.hat = array(0,c((nt-1),1))
	for(s in 0:(nt-2)){
		rh = NULL
		for(i in 1:ng){
			uh = uhat[((i-1)*nt+1):(i*nt)]
			q1 = (uh[1:(nt-s)]<0) - tau
			q2 = (uh[(1+s):nt]<0) - tau
			rh = c(rh,crossprod(q1,q2)/(nt-s))
		}
		rho.hat[s+1] = mean(rh)
	}
	rho.hat[1] = tau*(1-tau)	
	rhat.nb = rho.hat
	
	# 2. bias and long-run variance estimation	
	# 2.1. one-time bias correction
	vhat = rho.hat[1]+2*sum(((nt-(1:mt))/nt)*rho.hat[2:(mt+1)])
	bias.est = vhat/nt
	rho.hat.b1 = c(rho.hat[1],(rho.hat[-1] + bias.est))
	
	# 2.2. iterative bias correction
	b0 = bias.est
	rhat0 = rho.hat
	for(i in 1:10){ # iterate for 10 times
		b1 = long.var(rhat0,nt,mt)
		rhat1 = c(rho.hat[1],(rho.hat[-1] + b1))
		b0 = b1
		rhat0 = rhat1
	}
	longvar = nt*b1
	return(list(bias = b1, rho.hat = rhat1, rho.hat.nb = rhat.nb, v.hat = longvar))	
}

step.long <- function(nt,ng,hx,mt,ker){
	# long-run variance estimation
	# ker = 1,2,3 for truncated, Bartlett, and QS kernels
	Jm = hx[,,1]
	if(ker==1){ # truncated kernel
		for(s in 1:mt){
			Jm = Jm + 2*((nt-s)/nt)*hx[,,(s+1)]
		}
	}
	if(ker==2){ # Bartlett kernel
		for(s in 1:mt){
			Jm = Jm + 2*(1-abs(s/mt))*((nt-s)/nt)*hx[,,(s+1)]	
		}
	}
	if(ker==3){ # QS kernel
		for(s in 1:(nt-2)){
			z = s/mt
			Jm = Jm + 2*(25/(12*(pi^2)*(z^2)))*(sin(6*pi*z/5)/(6*pi*z/5)-cos(6*pi*z/5))*((nt-s)/nt)*hx[,,(s+1)]
		}
	}
	return(list(J.n = hx[,,1], J.r = Jm))
}

long.var <- function(rr,nt,m){
	vhat = rr[1]+2*sum(((nt-(1:m))/nt)*rr[2:(m+1)])
	bias.est = vhat/nt
	return(bias.est)
}

bias.cor <- function(hx,xx,v,nt,opt){
	if(opt==1){ # bias correction including s=0
		hx.c = hx + (v/nt)*xx	
	}
	if(opt==2){ # bias correction excluding s=0
		xx[,,1] <- 0
		hx.c = hx + (v/nt)*xx	
	}
	return(hx.c)
}

# 3. Functions that estimate the optimal bandwidth

band <- function(yy,nt,ng,dy,ker){
	if(dy==1){
		delta = ar1(yy,nt,ng)$delta
		if(ker %in% c(1,2)){
			ah = ((2*delta)/(1-(delta^2)))^2
			band = round((1.1447*((ah*nt*ng)^{1/3})),0)
		}
		if(ker %in% c(3)){
			ah = ((2*delta)/((1-delta)^{2}))^2
			band = round((1.3221*((ah*nt*ng)^{1/5})),0)
		}
	}
	if(dy>1){
		delta = array(0,c(dy,1))
		sig = array(0,c(dy,1))
		for(j in 1:dy){
			aa = ar1(yy[,j],nt,ng)
			delta[j] = aa$delta
			sig[j] = aa$sigma	
		}
		if(ker %in% c(1,2)){
			e1 = 4*(delta^2)*(sig^4)/(((1-delta)^6)*((1+delta)^2))
			e2 = (sig^4)/((1-delta)^4)
			ah = sum(e1)/sum(e2)
			band = round((1.1447*((ah*nt*ng)^{1/3})),0)			
		}
		if(ker %in% 3){
			e1 = 4*(delta^2)*(sig^4)/((1-delta)^8)
			e2 = (sig^4)/((1-delta)^4)
			ah = sum(e1)/sum(e2)
			band = round((1.3221*((ah*nt*ng)^{1/5})),0)			
		}
	}
	band = pmax(band,1)
	return(list(delta = delta, a.hat=ah, band=band))
}

ar1 <- function(yy,nt,ng){
	# estimates parameters for a AR(1) process 
	s1 <- 0; s2 <- 0
	for(i in 1:ng){
		sel = ((i-1)*nt+1):(i*nt)
		yh  = yy[sel]
		yh1 = yh[1:(nt-1)] - mean(yh[1:(nt-1)])
		s1 = s1 + sum(yh1*(yh[2:nt]-mean(yh[2:nt])))
		s2 = s2 + sum(yh1*yh1)
	}
	delta = (s1/s2)*nt/(nt-1) + 1/(nt-1)	
	er = NULL
	for(i in 1:ng){
		sel = ((i-1)*nt+1):(i*nt)
		yh = yy[sel]
		eh = yh[2:nt] - mean(yh[2:nt]) - delta*yh[1:(nt-1)]
		er = c(er,eh)	
	}
	sig = sd(er)
	return(list(delta = delta, sigma = sig))
}

# 4. Funcitons for the Score test

Sco.test <- function(x1,xe,uhat,tau,nt,ng,mt,bias,ker){
	# produce score statistics (i) without autocorrelation correction, (ii) with autocorrelation correction + bias correction
 	H  <- xe %*% solve(t(xe) %*% xe) %*% t(xe)
	xs <- x1 - (H %*% x1)
	Sn <- apply(xs*(tau - I(uhat<0)),2,sum)/sqrt(nt*ng)

	# 1. assume zero serial correlation
	Qn <- tau*(1-tau)*crossprod(xs,xs)/(nt*ng)		
	s.stat <- t(Sn) %*% solve(Qn) %*% Sn
		
	# 2. assume non-zero serial correlation
	if(is.matrix(x1)==0) {q = 1}
	if(is.matrix(x1)==1) {q = ncol(xs)}
	xx1 = array(0,c(q,q,(nt-1)))
	for(s in 0:(nt-2)){
		xx = 0; hx = 0
		for(i in 1:ng){
			sel = ((i-1)*nt+1):(i*nt)
			if(q==1){
				xh  = xs[sel]
				xh1 = xh[1:(nt-s)]
				xh2 = xh[(1+s):nt]				
			}
			if(q>1){
				xh  = xs[sel,]
				xh1 = xh[1:(nt-s),]
				xh2 = xh[(1+s):nt,]				
			}
			xx = xx + crossprod(xh1,xh2)/(nt-s)
		}
	xx1[,,(s+1)] = xx/ng
	}
	dd = step.one(nt,ng,uhat,tau,mt[1])	# bias estimation
	if(bias==0) {rhat = dd$rho.hat.nb}
	if(bias==1) {rhat = dd$rho.hat}
	Jm = rhat[1]*xx1[,,1]
	if(ker==1){ # truncated kernel
		for(s in 1:mt[2]){
			Jm = Jm + 2*((nt-s)/nt)*rhat[s+1]*xx1[,,(s+1)]
		}
	}
	if(ker==2){ # Bartlett kernel
		for(s in 1:mt[2]){
			Jm = Jm + 2*(1-abs(s/mt[2]))*((nt-s)/nt)*rhat[s+1]*xx1[,,(s+1)]
		}
	}
	if(ker==3){ # QS kernel
		for(s in 1:(nt-2)){
			z = s/mt[2]
			Jm = Jm + 2*(25/(12*(pi^2)*(z^2)))*(sin(6*pi*z/5)/(6*pi*z/5)-cos(6*pi*z/5))*((nt-s)/nt)*rhat[s+1]*xx1[,,(s+1)]
		}
	}
	s.stat.r <- t(Sn) %*% solve(Jm) %*% Sn
	return(list(score = s.stat, score.r = s.stat.r))
}

# 5. Functions that estimate the quantile regression coefficients

G<-function(x){
	if (x< -1) return (1) 
	if (abs(x) <= 1) return (0.5-(105/64)*(x-(5/3)*x^3+(7/5)*x^5-(3/7)*x^7)) 
	if (x>1) return (0) 
}

K<-function(x){
	if (abs(x) <= 1) return ((105/64)*(1-5*x^2+7*x^4-3*x^6))
	else return (0) 
}

dK<-function(x){
	if (abs(x) <= 1) return ((105/64)*(-10*x+28*x^3-18*x^5))
	else return (0)
}

Ku<-function(x){
	if (abs(x) <= 1) return (x*(105/64)*(1-5*x^2+7*x^4-3*x^6))
	else return (0)
}

dKu<-function(x){
	if (abs(x) <= 1) return (x*(105/64)*(-10*x+28*x^3-18*x^5))
	else return (0)
}

rq.opt2 <- function(coefs,tau,hh,y,x,D,n,t) { 
	# this function defines the objective function of the smoothed QR
	if(is.matrix(x)==0){
		ehat <- y-D %*%coefs[1:n]-coefs[(n+1)]*x
		}
	if(is.matrix(x)==1){
		p <- ncol(x)
		ehat<-y-D%*%coefs[1:n]-x%*%coefs[(n+1):(n+p)]
		}
	Gh <- apply(ehat/hh,1,G)
	obj <- (1/n*t)*sum(ehat*(tau-Gh))  
	return(obj)
}

rq.bsqr <- function(ehat,bn,x,beta0,ng,nt,tau){
	# this function implements the analytical bias correction method
	x = as.matrix(x)
	dx = ncol(x)
	fi.aux<-apply(ehat/bn,1,K)
	nui.aux.1<-apply(ehat/bn,1,dK)
	beta.s <- array(0,c(dx,1))	# bias corrected smoothed QR
	for(j in 1:dx){
	gammai.aux<-apply(ehat/bn,1,K)*x[,j]
	nui.aux.2<-apply(ehat/bn,1,dK)*x[,j]
	Gamma.aux<-apply(ehat/bn,1,K)*x[,j]
	
	si<-matrix(0,ng,1)
	gammai<-matrix(0,ng,1)
	nui<-matrix(0,ng,1)
	fi<-matrix(0,ng,1)
	for (i in 1:ng){
		fi[i]<-(1/(nt*bn))*sum(fi.aux[(1+(i-1)*nt):(i*nt)])
		if(fi[i]>0.01){
			si[i] <- 1/fi[i]
			gammai[i] <- si[i]*((1/(nt*bn))*sum(gammai.aux[(1+(i-1)*nt):(i*nt)]))
			nui[i] <- (1/(bn^(2)*nt))*(sum(nui.aux.2[(1+(i-1)*nt):(i*nt)])-sum(nui.aux.1[(1+(i-1)*nt):(i*nt)])*gammai[i]) 
		}
		else{
			Gamma.aux[(1+(i-1)*nt):(i*nt)]<-rep(0,nt)
		}
	}
	Gamma<-(1/(ng*nt*bn))*sum(Gamma.aux*(x[,j] - rep(gammai,rep(nt,ng))))

	b.hat <- (Gamma^(-1))*(((tau*(1-tau))/2)*(1/ng)*sum(si^2*nui))
	beta.s[j] <- beta0[j] - b.hat/nt
	}
	return(beta.s)
}
