## Replication file for 
## "Cluster Robust Covariance Matrix Estimation in Panel Quantile Regression with Individual Fixed Effects"
## by Jungmo Yoon and Antonio Galvao.
## This R code reproduces simualtion results, Power of tests, in the Online Supplemetal Appendix.
## See 'run_simulation_size.R' for further explanations.

rm(list=ls())
require(quantreg)
source("ccm_fn.R")
load("sd_Y_under_null_2.rda")

tt = 0.5	# quantile level, try 0.25 and 0.75 as well
model = 2	# choice of models, 1 location shift, 2 location scale, 3 rho different for each i, 4 nonlinear heteroskedasticity term
est.opt = 2	# estimator for regression coefficients; 1 use FE-QR, 2 analytical correction, 3 split sample jackknife

L = 1000		# simulation repetitions
ker = 2		# kernels, 1 truncated, 2 Bartlett, 3 QS kernels

if(model %in% c(1,4)){gamma0 = 0}
if(model %in% c(2,3)){gamma0 = 0.75}
rho.x = rep(0.8,2)
const1 <- 1; const2 <- 2	# constants for the smoothed QR

# (N,T) combination
if(model %in% c(1,2,4)){parm = cbind(rep(c(50,50,20,100),each=3),rep(c(20,50,100,20),each=3),rep(c(0.0,0.4,0.9),4))}
if(model==3){parm = cbind(rep(c(50,50,20,100),each=2),rep(c(20,50,100,20),each=2),rep(c(0,1),4))}

Rt <- array(0,c(3,2,nrow(parm)))	# Save test statistics

#for(k in 1:nrow(parm)){	# iteration over (N,T) combination
for(k in 6:6){	# one (N,T) at a time 

ng = parm[k,1]		# number of crosssecrtional units 
nt = parm[k,2]		# length of years
rho = parm[k,3]		# AR(1) coefficient of error terms
con = sdY[k,model]	# determines the size of the effects under H_1

EH <- array(0,c(L,3))	# save Score test
WD <- array(0,c(L,3))	# save Wald test
for(jj in 1:3){	# repetition over the size of effects under the alternative

theta = cbind((c(0.2,0.4,0.6)*con),rep(1,3))
beta = theta[jj,]
for(ll in 1:L){
	
	# 1. data generation
	
	# 1.1. fixed effects
	alp0 = runif(ng,min=-1,max=1)
	state = rep((1:ng),each=nt)
	year = rep((1:nt),ng)

	# 1.2. treatment variable and control variable

	yr = unique(year)
	st = unique(state)
	x.t <- sample(yr[(round(nt/4)+1):(round(3*nt/4)-1)],1)
	x.s <- sort(sample(st,round(ng/2),replace=FALSE))
	x1 <- ((year >= x.t) & (state%in%x.s))		

	x2 <- NULL
	for(i in 1:ng){
		err <- rnorm(nt,mean=0,sd=sqrt(1-(rho.x[2]^2)))
		err <- err^2	
		zz  <- array(0,c(nt,1))
		zz[1] <- err[1]
		for(j in 2:nt){
			zz[j] <- rho.x[2]*zz[(j-1)] + err[j]
		}
		x2 <- c(x2,zz)
	}
	x2 <- 0.3*rep(alp0,each=nt) + x2
	x = cbind(x1,x2)
	dx = ncol(x)
	
	# 1.3. error term
	u = NULL	
	if(model %in% c(1,2)){
	for(i in 1:ng){
		e = rnorm(nt,mean=0,sd=sqrt(1-rho^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rho*v0 + e[j]}
			else {v[j] = rho*v[j-1]+e[j]}
		}
	u = c(u,v)
	}
	}
	if(model %in% 3){
	for(i in 1:ng){
		if(rho==0){rhoi = runif(1,min=0,max=0.5)}
		if(rho==1){rhoi = runif(1,min=0.5,max=1.0)}
		e = rnorm(nt,mean=0,sd=sqrt(1-rhoi^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rhoi*v0 + e[j]}
			else {v[j] = rhoi*v[j-1]+e[j]}
		}
		u <- c(u,v)
	}
	}
	if(model==4){
	for(i in 1:ng){
		het = x2[((i-1)*nt+1):(i*nt)]
		e = rnorm(nt,mean=0,sd=sqrt(1-rho^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rho*v0 + e[j]}
			else {v[j] = rho*v[j-1] + sqrt(0.5+0.5*(het[j]^2))*e[j]}
		}
		u <- c(u,v)
	}
	}
	u <- u - qnorm(tt)
	
	# 1.4. Dependent variable
	y = (alp0 %x% rep(1,nt)) + beta[1]*x1 + beta[2]*x2 + (1+gamma0*x2)*u
	y = as.vector(y)
	data = as.data.frame(cbind(state,year,x1,x2,y))
	
	# 2. Estimating regression coefficients 
	# 2.1. FE-QR
	a1 = rq(y~factor(state)+x1+x2-1,tau=tt,data=data)
	uhat = a1$resid
	beta.fe <- a1$coef[(ng+1):(ng+dx)]

	# 2.2. FE-SQR
	if(est.opt %in% c(2,3)){
		D <- diag(ng) %x% rep(1,nt) # dummies for states
		s1 <- sd(uhat)
		hn <- const1*s1*(ng*nt)^(-1/7)
		fit.sqr <- optim(a1$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x,D=D,n=ng,t=nt)
		beta.sqr <- fit.sqr$par[(ng+1):(ng+dx)]
	}
	
	# 2.3. Analytical bias corrected estimator
	if(est.opt==2){
		ehat.f <- y-D%*%fit.sqr$par[1:ng]-x%*%fit.sqr$par[(ng+1):(ng+dx)]
		s2 <- sd(ehat.f)
		bn <- const2*s2*nt^(-1/5)
		beta.bsqr <- rq.bsqr(ehat.f,bn,x,beta.sqr,ng,nt,tt)
	}
	
	# 2.4. Jackknife
	if(est.opt==3){
		D.S = diag(ng) %x% rep(1,nt/2) # dummies for states
		ind.S1 = rep(rep(c(1,0),each=(nt/2)),ng)
		ind.S2 = rep(rep(c(0,1),each=(nt/2)),ng)
		fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x[(ind.S1==1),],D=D.S,n=ng,t=(nt/2))
		fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x[(ind.S2==1),],D=D.S,n=ng,t=(nt/2))
		beta.sqr.S1 <- fit.sqr.S1$par
		beta.sqr.S2 <- fit.sqr.S2$par
		beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx)]/2
	}
	if(est.opt==2){
		#Co[ll,,2] <- beta.bsqr
		uhat <- y-D%*%fit.sqr$par[1:ng]-x%*%beta.bsqr
		uhat <- as.vector(uhat)
	}
	if(est.opt==3){
		#Co[ll,,3] <- beta.jack
		coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
		uhat <- y-D%*%coef.jack[1:ng]-x%*%coef.jack[(ng+1):(ng+dx)]
		uhat <- as.vector(uhat)
	}
	
	# 3. Estimating Auto-covariance
	
	st.dummy = diag(ng) %x% rep(1,nt) 
	x.ext = cbind(st.dummy,x1,x2)

	# 3.1. bandwidth esitmation
	
	mt = array(0,c(2,1))	
	# estimate the bandwidths	
	h = (tt - I(uhat<0))
	dd = ar1(h,nt,ng)$delta
	mt[1] <- band(h,nt,ng,dy=1,ker)$band
	xd = array(0,dim(x))		# demeaned x
		for(i in 1:ng){
			sel <- ((i-1)*nt+1):(i*nt)
			xh <- x[sel,]
			xd[sel,] <- xh - (t(apply(xh,2,mean)) %x% rep(1,nt))
		}
	H2 = h*xd
	mt[2] <- band(H2,nt,ng,dy=ncol(H2),ker)$band
	mt2 <- rep(mt[1],2)
	mt <- pmin(pmax(mt,1),(nt-2))
	mt2 <- pmin(pmax(mt2,1),(nt-2))

	# 3.2. CCM estimators
	
	bb  <- summary(a1,se="ker",covariance=T)
	bb3 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=1,bias=1,ker)
	se <- sqrt(diag(bb3$cov))[(ng+1):(ng+2)]

	# 4.1. Wald tests
	if(est.opt==1){theta = a1$coef[ng+1]}
	if(est.opt==2){theta = beta.bsqr[1]}
	if(est.opt==3){theta = beta.jack[1]}
	WD[ll,jj] <- (theta/se[1])^2

	# 4.2. Score tests
	x.ext2 <- cbind(st.dummy,x2)
	dx2 = dx-1

	# 4.2.1. Regression coefficients
	a2 <- rq(y~factor(state)+x2-1,tau=tt,data=data)
	uhat2  <- a2$resid
	
	if(est.opt %in% c(2,3)){
		s1 <- sd(uhat2)
		hn <- const1*s1*(ng*nt)^(-1/7)
		fit.sqr <- optim(a2$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x2,D=D,n=ng,t=nt)
		beta.sqr <- fit.sqr$par[(ng+1):(ng+dx2)]
	}
	# Analytical bias corrected estimator
	if(est.opt==2){
		ehat.f <- y-D%*%fit.sqr$par[1:ng]-x2*fit.sqr$par[(ng+1):(ng+dx2)]
		s2 <- sd(ehat.f)
		bn <- const2*s2*nt^(-1/5)
		beta.bsqr <- rq.bsqr(ehat.f,bn,x2,beta.sqr,ng,nt,tt)
		uhat2 <- y-D%*%fit.sqr$par[1:ng]-x2*beta.bsqr
		uhat2 <- as.vector(uhat2)
	}
	# Jackknife
	if(est.opt==3){	
		fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x2[(ind.S1==1)],D=D.S,n=ng,t=(nt/2))
		fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x2[(ind.S2==1)],D=D.S,n=ng,t=(nt/2))
		beta.sqr.S1 <- fit.sqr.S1$par
		beta.sqr.S2 <- fit.sqr.S2$par
		beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx2)]/2
		coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
		uhat2 <- y-D%*%coef.jack[1:ng]-x2*coef.jack[(ng+1):(ng+dx2)]
		uhat2 <- as.vector(uhat2)	
	}
	
	# 4.2.2. Score tests	
	sc2 <- Sco.test(x1,x.ext2,uhat2,tau=tt,nt,ng,mt2,bias=1,ker=1)
	EH[ll,jj] <- sc2$score.r
	
	# 4.3. collect bandwidths
	print("the number of iteration is "); print(ll)
	} # inner loop
	print("the size of beta is "); print(jj)
} # loop for different beta's

# Calculate Power of Score and Wald tests
R2 <- array(0,c(3,2))
crit <- qchisq(0.95,df=1)
for(i in 1:3){
	R2[i,1] <- mean(EH[,i] > crit)
	R2[i,2] <- mean(WD[,i] > crit)
}
Rt[,,k] <- R2
} # iteration for simulation set-up

save(parm,Rt,model,tt,L,ker,est.opt,rho.x,gamma0,file=paste("ccm_power","_model",model,"_est",est.opt,"_ker",ker,"_tau",tt,".rda",sep=""))
