## Replication file for 
## "Cluster Robust Covariance Matrix Estimation in Panel Quantile Regression with Individual Fixed Effects"
## by Jungmo Yoon and Antonio Galvao.
## This R code reproduces simualtion results in the Online Supplemetal Appendix.
## You may choose a model (M1,M2,M3,M4), a quantile level (tau), and an estimator for the QR coefficients. 
## Repeat this over all configurations will reproduce the simulation section in the Supplemetal Appendix.
## The following code runs over many choices of (N,T). To save time, it is a good idea to choose a particular (N,T) value at a time. 

rm(list=ls())
require(quantreg)
source("ccm_fn.R")

tt = 0.5	# quantile level, try 0.25 and 0.75 as well
model = 1	# 1 means the location shift model, 2 location scale, 3 rho different for each i, 4 nonlinear heteroskedasticity term
est.opt = 2	# 1 is for the FE-QR, 2 the analytical bias correction estimator, 3 the split sample jackknife estimator

L = 2000	# number of simulation repetitions
ker = 1		# choice of kernel; 1 truncated, 2 Bartlett, 3 QS kernels
wald.opt = 2	# how to do Wald test? choose 1 for a joint hypothesis (change a matrix 'R' accordingly)

beta = c(0.0,1.0)
if(model %in% c(1,4)){gamma0 = 0}
if(model %in% c(2,3)){gamma0 = 0.75}
rho.x = rep(0.8,2)
const1 <- 1; const2 <- 2	# constants for the smoothed QR estimator

# (N,T) combination
if(model %in% c(1,2,4)){parm = cbind(rep(c(50,50,20,100),each=3),rep(c(20,50,100,20),each=3),rep(c(0.0,0.4,0.9),4))}
if(model==3){parm = cbind(rep(c(50,50,20,100),each=2),rep(c(20,50,100,20),each=2),rep(c(0,1),4))}
# (N,T) additional trial
#parm = cbind(rep(400,3),rep(30,3),c(0.0,0.4,0.9))

Rc = array(0,c(5,2,nrow(parm)))		# save coverage rates of confidence intervals
Rt <- array(0,c(3,2,nrow(parm)))	# save Score and Wald tests
Bm <- array(0,c(4,2,nrow(parm)))	# save Bandwidth
Ca <- array(0,c(3,2,nrow(parm)))	# save regression coefficients estimates (FE-QR)
Ce <- array(0,c(3,2,nrow(parm)))	# save regression coefficients estimates (Bias corrected)

#for(k in 1:nrow(parm)){	# runs for all (N,T) combination
for(k in 3:3){	# choose one particular (N,T) value
		
ng = parm[k,1]		# number of groups
nt = parm[k,2]		# length of time-series
rho = parm[k,3]		# AR(1) coefficient of error terms

A <- array(0,c(L,2))	# save FE-QR coefficients	
B <- array(0,c(L,2,4))	# save standard errors
B2 <- array(0,c(L,2))	# save standard errors from summary.rq()
EH <- array(0,c(L,3))	# save Score test
WD <- array(0,c(L,3))	# save Wald test
BW1 <- array(0,c(L,2))	# save bandwidth for the CCM
BW2 <- array(0,c(L,2))	# save bandwidth for the simplified CCM
Co <- array(0,c(L,2,4))	# save regression coefficients estimates

for(ll in 1:L){	# iteration over simulation repetitions
	
	# 1. data generation
	
	# 1.1. fixed effects
	alp0 = runif(ng,min=-1,max=1)
	state = rep((1:ng),each=nt)
	year = rep((1:nt),ng)

	# 1.2. treatment variable 
	yr = unique(year)
	st = unique(state)
	x.t <- sample(yr[(round(nt/4)+1):(round(3*nt/4)-1)],1)
	x.s <- sort(sample(st,round(ng/2),replace=FALSE))
	x1 <- ((year >= x.t) & (state%in%x.s))		
	
	# 1.3. other control variable
	x2 <- NULL
	for(i in 1:ng){
		err <- rnorm(nt,mean=0,sd=sqrt(1-(rho.x[2]^2)))
		err <- err^2	
		zz  <- array(0,c(nt,1))
		zz[1] <- err[1]
		for(j in 2:nt){
			zz[j] <- rho.x[2]*zz[(j-1)] + err[j]
		}
		x2 <- c(x2,zz)
	}
	x2 <- 0.3*rep(alp0,each=nt) + x2

	x = cbind(x1,x2)
	dx = ncol(x)
	
	# 1.4. error terms (model specific)
	u = NULL	
	if(model %in% c(1,2)){
	for(i in 1:ng){
		e = rnorm(nt,mean=0,sd=sqrt(1-rho^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rho*v0 + e[j]}
			else {v[j] = rho*v[j-1]+e[j]}
		}
	u = c(u,v)
	}
	}
	if(model %in% 3){
	for(i in 1:ng){
		if(rho==0){rhoi = runif(1,min=0,max=0.5)}
		if(rho==1){rhoi = runif(1,min=0.5,max=1.0)}
		e = rnorm(nt,mean=0,sd=sqrt(1-rhoi^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rhoi*v0 + e[j]}
			else {v[j] = rhoi*v[j-1]+e[j]}
		}
		u <- c(u,v)
	}
	}
	if(model==4){
	for(i in 1:ng){
		het = x2[((i-1)*nt+1):(i*nt)]
		e = rnorm(nt,mean=0,sd=sqrt(1-rho^2))
		v = array(0,c(nt,1))
		v0 = rnorm(1)
		for(j in 1:nt){
			if(j==1) {v[j] = rho*v0 + e[j]}
			else {v[j] = rho*v[j-1] + sqrt(0.5+0.5*(het[j]^2))*e[j]}
		}
		u <- c(u,v)
	}
	}
	u <- u - qnorm(tt)	# centering of the error
	
	# 1.5. Dependent variable
	y = (alp0 %x% rep(1,nt)) + beta[1]*x1 + beta[2]*x2 + (1+gamma0*x2)*u
	y = as.vector(y)
	data = as.data.frame(cbind(state,year,x1,x2,y))
	
	# 2. Estimating quantile regression coefficients 
	# 2.1. FE-QR
	a1 = rq(y~factor(state)+x1+x2-1,tau=tt,data=data)
	uhat = a1$resid
	beta.fe <- a1$coef[(ng+1):(ng+dx)]

	# 2.2. FE-SQR
	if(est.opt %in% c(2,3)){
		D <- diag(ng) %x% rep(1,nt) # dummies for states
		s1 <- sd(uhat)
		hn <- const1*s1*(ng*nt)^(-1/7)
		fit.sqr <- optim(a1$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x,D=D,n=ng,t=nt)
		beta.sqr <- fit.sqr$par[(ng+1):(ng+dx)]
	}
	
	# 2.3. Analytical bias corrected estimator
	if(est.opt==2){
		ehat.f <- y-D%*%fit.sqr$par[1:ng]-x%*%fit.sqr$par[(ng+1):(ng+dx)]
		s2 <- sd(ehat.f)
		bn <- const2*s2*nt^(-1/5)
		beta.bsqr <- rq.bsqr(ehat.f,bn,x,beta.sqr,ng,nt,tt)
	}
	
	# 2.4. Jackknife estimator
	if(est.opt==3){
		D.S <- diag(ng) %x% rep(1,nt/2) # dummies for states
		ind.S1 <- rep(rep(c(1,0),each=(nt/2)),ng)
		ind.S2 <- rep(rep(c(0,1),each=(nt/2)),ng)
		fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x[(ind.S1==1),],D=D.S,n=ng,t=(nt/2))
		fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x[(ind.S2==1),],D=D.S,n=ng,t=(nt/2))
		beta.sqr.S1 <- fit.sqr.S1$par
		beta.sqr.S2 <- fit.sqr.S2$par
		beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx)]/2
	}
	
	# 2.5. save outcomes
	A[ll,] <- beta.fe
	Co[ll,,1] <- beta.fe
	if(est.opt %in% c(2,3)){
		Co[ll,,2] <- beta.sqr
		}
	if(est.opt==2){
		Co[ll,,2] <- beta.bsqr
		uhat <- y-D%*%fit.sqr$par[1:ng]-x%*%beta.bsqr
		uhat <- as.vector(uhat)
	}
	if(est.opt==3){
		Co[ll,,3] <- beta.jack
		coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
		uhat <- y-D%*%coef.jack[1:ng]-x%*%coef.jack[(ng+1):(ng+dx)]
		uhat <- as.vector(uhat)
	}
	
	# 3. Estimating Auto-covariance matrices
	
	st.dummy = diag(ng) %x% rep(1,nt) 
	x.ext = cbind(st.dummy,x1,x2)

	# 3.1. bandwidth esitmation
	
	mt = array(0,c(2,1))	
	# estimate the bandwidths	
	h = (tt - I(uhat<0))
	dd = ar1(h,nt,ng)$delta
	mt[1] <- band(h,nt,ng,dy=1,ker)$band
	xd = array(0,dim(x))		# demeaned x
		for(i in 1:ng){
			sel <- ((i-1)*nt+1):(i*nt)
			xh <- x[sel,]
			xd[sel,] <- xh - (t(apply(xh,2,mean)) %x% rep(1,nt))
		}
	H2 = h*xd
	mt[2] <- band(H2,nt,ng,dy=ncol(H2),ker)$band
	mt <- pmin(pmax(mt,1),(nt-2))
	# bandwidth for the simplified CCM
	mt2 <- rep(mt[1],2)
	mt2 <- pmin(pmax(mt2,1),(nt-2))

	# 3.2. CCM estimators
	
	bb  <- summary(a1,se="ker",covariance=T)
	#bb1 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=0,bias=0,ker)
	bb2 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=1,bias=0,ker)
	bb3 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=1,bias=1,ker)
	bb4 <- ccm.se.s(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt2,se=1,bias=1,ker=1)
	#B[ll,,1] <- sqrt(diag(bb1$cov))[(ng+1):(ng+2)]
	B[ll,,1] <- sqrt(diag(bb2$covs))[(ng+1):(ng+2)]
	B[ll,,2] <- sqrt(diag(bb2$cov))[(ng+1):(ng+2)]
	B[ll,,3] <- sqrt(diag(bb3$cov))[(ng+1):(ng+2)]
	B[ll,,4] <- sqrt(diag(bb4$cov))[(ng+1):(ng+2)]
	B2[ll,] <- bb$coef[(ng+1):(ng+2),2]

	# 4. Wald tests
	
	if(wald.opt==1){
		R = c(rep(0,ng),1,0)
		if(est.opt==1){theta = a1$coef}
		if(est.opt==2){theta = c(fit.sqr$par[1:ng],beta.bsqr)}
		if(est.opt==3){theta = coef.jack}
		WD[ll,1] <- (t(R) %*% theta) %*% solve(t(R) %*% bb2$covs %*% R) %*% (t(R) %*% theta)
		WD[ll,2] <- (t(R) %*% theta) %*% solve(t(R) %*% bb2$cov %*% R) %*% (t(R) %*% theta)
		WD[ll,3] <- (t(R) %*% theta) %*% solve(t(R) %*% bb3$cov %*% R) %*% (t(R) %*% theta)
	}
	if(wald.opt==2){
		if(est.opt==1){theta = a1$coef[ng+1]}
		if(est.opt==2){theta = beta.bsqr[1]}
		if(est.opt==3){theta = beta.jack[1]}
		WD[ll,1] <- (theta/B[ll,1,1])^2
		WD[ll,2] <- (theta/B[ll,1,2])^2
		WD[ll,3] <- (theta/B[ll,1,3])^2
	}

	# 5. Score tests
	x.ext2 <- cbind(st.dummy,x2)
	dx2 = dx-1

	# 5.1. QR coefficients estimation under the null
	a2 <- rq(y~factor(state)+x2-1,tau=tt,data=data)
	uhat2  <- a2$resid
	
	if(est.opt %in% c(2,3)){
		s1 <- sd(uhat2)
		hn <- const1*s1*(ng*nt)^(-1/7)
		fit.sqr <- optim(a2$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x2,D=D,n=ng,t=nt)
		beta.sqr <- fit.sqr$par[(ng+1):(ng+dx2)]
	}
	# Analytical bias corrected estimator
	if(est.opt==2){
		ehat.f <- y-D%*%fit.sqr$par[1:ng]-x2*fit.sqr$par[(ng+1):(ng+dx2)]
		s2 <- sd(ehat.f)
		bn <- const2*s2*nt^(-1/5)
		beta.bsqr <- rq.bsqr(ehat.f,bn,x2,beta.sqr,ng,nt,tt)
		uhat2 <- y-D%*%fit.sqr$par[1:ng]-x2*beta.bsqr
		uhat2 <- as.vector(uhat2)
	}
	# Jackknife estimator
	if(est.opt==3){	
		fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x2[(ind.S1==1)],D=D.S,n=ng,t=(nt/2))
		fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x2[(ind.S2==1)],D=D.S,n=ng,t=(nt/2))
		beta.sqr.S1 <- fit.sqr.S1$par
		beta.sqr.S2 <- fit.sqr.S2$par
		beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx2)]/2
		coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
		uhat2 <- y-D%*%coef.jack[1:ng]-x2*coef.jack[(ng+1):(ng+dx2)]
		uhat2 <- as.vector(uhat2)	
	}
	
	# 5.2. Score tests	
	sc  <- Sco.test(x1,x.ext2,uhat2,tau=tt,nt,ng,mt2,bias=0,ker=1)
	sc2 <- Sco.test(x1,x.ext2,uhat2,tau=tt,nt,ng,mt2,bias=1,ker=1)
	EH[ll,1] <- sc$score
	EH[ll,2] <- sc$score.r
	EH[ll,3] <- sc2$score.r
	
	# 6. Collect bandwidths
	BW1[ll,] <- mt; BW2[ll,] <- mt2
	print("the number of iteration is "); print(ll)
} # inner loop

# 7. Calculates coverage ratios, rejection frequencies, and other statistics

# Bias and RMSE of beta.hat
M = array(0,c(3,2))
for(i in 1:2){
	diff <- A[,i] - beta[i]
	M[1,i] <- mean(diff) # bias
	M[2,i] <- sqrt(crossprod(diff)/L) # RMSE
	M[3,i] <- sd(A[,i]) # srandard deviation
}

# All four estimators for beta.hat
MA = array(0,c(3,2,4))
for(j in 1:4){
for(i in 1:2){
	diff <- Co[,i,j] - beta[i]
	MA[1,i,j] <- mean(diff) # bias
	MA[2,i,j] <- sqrt(crossprod(diff)/L) # RMSE
	MA[3,i,j] <- sd(Co[,i,j]) # srandard deviation
}
}

# Means and Standard deviations of Standard errors
Q = array(0,c(2,4,2))
for(i in 1:2){
	Q[i,,1] <- apply(B[,i,],2,mean)
	Q[i,,2] <- apply(B[,i,],2,sd)
}

# Coverage ratios of confidnece intervals
R = array(0,c(5,2))
for(j in 1:4){
	for(i in 1:2){ # i=1,2 means beta1 and beta2
		R[j,i] <- mean(abs(Co[,i,est.opt] - beta[i]) < 1.96*B[,i,j])
	}
}
for(i in 1:2){ # size of the defualt rq() standard error
	R[5,i] <- mean(abs(Co[,i,est.opt] - beta[i]) < 1.96*B2[,i])
}

# Rejection frequency of Score and Wald tests
R2 <- array(0,c(3,2))
crit <- qchisq(0.95,df=1)
for(i in 1:3){
	R2[i,1] <- mean(EH[,i] > crit)
	R2[i,2] <- mean(WD[,i] > crit)
}

# Mean and standard deviation of bandwidths
bm1 = c(apply(BW1,2,mean),apply(BW1,2,sd))
bm2 = c(apply(BW2,2,mean),apply(BW2,2,sd))

Rc[,,k] <- R
Rt[,,k] <- R2
Bm[,1,k] <- bm1
Bm[,2,k] <- bm2
Ca[,,k] <- M
Ce[,,k] <- MA[,,est.opt]
} # iteration for the simulation set-up

save(parm,Rc,Rt,Bm,Ca,Ce,model,tt,L,ker,est.opt,beta,rho.x,gamma0,file=paste("ccm","_model",model,"_est",est.opt,"_ker",ker,"_tau",tt,".rda",sep=""))
