## Replication file for 
## "Cluster Robust Covariance Matrix Estimation in Panel Quantile Regression with Individual Fixed Effects"
## by Jungmo Yoon and Antonio Galvao
## This R code replicates the states unemployment example in the paper.
## See 'run_simulation_size.R' for further explanations.

rm(list=ls())
library(quantreg)
source("ccm_fn.R")

tt.s <- c(0.5,0.75)		# quantile level
LL <- 2000				# number of simulation iterations

## Read data and construct variables

data <- read.csv(file="state_all.csv")			# state-level data
data.nat <- read.csv(file="national_all.csv")	# national-level data
D <- data[(data$Month=="Annual"),]
D <- D[,-4]

state <- unique(D$state.code)
state.name <- unique(D$state)
year <- unique(D$Year)
ng <- length(state)		# number of states
nt <- length(year)		# number of years
nt.s <- round(nt/2,0)	# number of states for the split sample jackknife
est.opt = 2		# estimation method for QR coefficients; 1 for FE-QR, 2 for analytical bias correction, 3 for split sample jackknife
ker = 1			# choice of kernels; 1 for truncated, 2 for Bartlett, 3 for QS
const1 <- 1; const2 <- 2	# constants for the smoothed QR

unemp <- NULL
for(i in 1:ng){
	unemp <- c(unemp,D$unemployment.rate[(D$state.code==state[i])] - data.nat$Unemployment.Rate)
}
D$dunemployment <- unemp

# AR coefficients of unemployment rates 
rho.st <- NULL
for(i in 1:ng){
	ss <- D$dunemployment[(D$state.code==state[i])]
	rho1 <- crossprod(ss[-nt],ss[-1])/crossprod(ss[-nt],ss[-nt])
	rho2 <- crossprod(ss[-c((nt-1),nt)],ss[-c(1,2)])/crossprod(ss[-c((nt-1),nt)],ss[-c((nt-1),nt)])
	rho3 <- crossprod(ss[-c((nt-2),(nt-1),nt)],ss[-c(1,2,3)])/crossprod(ss[-c((nt-2),(nt-1),nt)],ss[-c((nt-2),(nt-1),nt)])
	rho <- c(rho1,rho2,rho3)
	rho.st <- rbind(rho.st,rho)
}

# make state and year fixed effects
data.p <- as.data.frame(cbind(D$state.code,D$Year))
colnames(data.p) <- c("state","year")
for(t in sort(state)) { # state dummy
	data.p[paste("state",t,sep="")] <- ifelse(data.p$state==t,1,0)
	}	
for(t in sort(year)) { # year dummy
	data.p[paste("year",t,sep="")] <- ifelse(data.p$year==t,1,0)
	}	

y <- D$dunemployment
x1 <- log(D$employment/100000)

## Simulation Exercise

#for(l in 1:length(tt.s)){	# iteration over tau
for(l in 1:1){	# run one tau at a time
tt = tt.s[l]

tstat.ols<- NULL; tstat.rq1<- NULL; tstat.rq2<- NULL
scostat  <- NULL; scostat2 <- NULL; scostat3 <- NULL
waldstat <- NULL; waldstat2<- NULL; waldstat3<- NULL
coef.rq <- NULL
for(j in 1:LL){	# loop over simulation repetitions

# 1.1. treatment variable, generate random fictitious laws
treat.s <- sample(state,25,replace=FALSE)
treat.t <- rep(sample(seq(1985,2002,by=1),ng,replace=TRUE),each=nt)
treat <- ((data.p$year >= treat.t) & (data.p$state%in%treat.s))	

x <- as.matrix(cbind(treat,x1,data.p[,3:(ng+2)]))

# 1.2. OLS
ols <- lm(y~treat+x1+factor(data.p$state))
tstat.ols <- c(tstat.ols,summary(ols)$coef[2,3])

# 1.3. Conventional QR by rq() function
qr <- rq(y~treat+x1+factor(data.p$state),tau=tt)
tstat.rq1 <- c(tstat.rq1,summary(qr,se="iid")$coef[2,3])
tstat.rq2 <- c(tstat.rq2,summary(qr,se="ker")$coef[2,3])

# 2.1. estimating the model using FE-QR

a1 = rq(y~factor(state)+treat+x1-1,tau=tt,data=data.p)
uhat = a1$resid
x.s = cbind(treat,x1)
dx = ncol(x.s)
x.ext = cbind(x[,3:(ng+2)],x.s)
beta.fe <- a1$coef[(ng+1):(ng+dx)]

# 2.2. estimating the model using FE-SQR
if(est.opt %in% c(2,3)){
	D1 = diag(ng) %x% rep(1,nt) # dummies for states
	s1 <- sd(uhat)
	hn <- const1*s1*(ng*nt)^(-1/7)
	fit.sqr <- optim(a1$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x.s,D=D1,n=ng,t=nt)
	beta.sqr <- fit.sqr$par[(ng+1):(ng+dx)]
}
	
# 2.3. Analytical bias corrected estimator
if(est.opt==2){
	ehat.f <- y-D1%*%fit.sqr$par[1:ng]-x.s%*%fit.sqr$par[(ng+1):(ng+dx)]
	s2 <- sd(ehat.f)
	bn <- const2*s2*nt^(-1/5)
	beta.bsqr <- rq.bsqr(ehat.f,bn,x.s,beta.sqr,ng,nt,tt)
	uhat <- y-D1%*%fit.sqr$par[1:ng]-x.s%*%beta.bsqr
	uhat <- as.vector(uhat)	
}

# 2.4. Jackknife estimator
if(est.opt==3){
	D.S = diag(ng) %x% rep(1,nt.s) # dummies for states
	ind.S1 = c(rep(rep(c(1,0),each=(nt.s)),ng),rep(0,ng))
	ind.S2 = c(rep(rep(c(0,1),each=(nt.s)),ng),rep(0,ng))
	fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x.s[(ind.S1==1),],D=D.S,n=ng,t=(nt.s))
	fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x.s[(ind.S2==1),],D=D.S,n=ng,t=(nt.s))
	beta.sqr.S1 <- fit.sqr.S1$par
	beta.sqr.S2 <- fit.sqr.S2$par
	beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx)]/2
	coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
	uhat <- y-D1%*%coef.jack[1:ng]-x.s%*%coef.jack[(ng+1):(ng+dx)]
	uhat <- as.vector(uhat)
}

# 3. Testing

# 3.1. Optimal bandwidth estimation
	mt <- array(0,c(2,1))
	h = (tt - I(uhat<0))
	dd = ar1(h,nt,ng)$delta
	mt[1] <- band(h,nt,ng,dy=1,ker)$band
	xd = array(0,dim(x.s))		# demeaned x
	for(i in 1:ng){
		sel <- ((i-1)*nt+1):(i*nt)
		xh <- x.s[sel,]
		xd[sel,] <- xh - (t(apply(xh,2,mean)) %x% rep(1,nt))
	}
	H2 = h*xd
	mt[2] <- band(H2,nt,ng,dy=ncol(H2),ker)$band
	mt2 <- rep(mt[1],2)
	mt <- pmin(pmax(mt,1),(nt-2))
	mt2 <- pmin(pmax(mt2,1),(nt-2))

# 3.2. Variance-Covariance Matrix estimation by CCM estimator
	bb  <- summary(a1,se="ker",covariance=T)
	#bb1 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=0,bias=0,ker)
	bb2 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=1,bias=0,ker)
	bb3 <- ccm.se(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt,se=1,bias=1,ker)
	bb4 <- ccm.se.s(x.ext,(tt*(1-tt)*bb$J),bb$Hinv,uhat,tau=tt,nt,ng,mt2,se=1,bias=1,ker=1)
	se1 <- sqrt(diag(bb2$covs))[(ng+1):(ng+2)]
	se2 <- sqrt(diag(bb2$cov))[(ng+1):(ng+2)]
	se3 <- sqrt(diag(bb3$cov))[(ng+1):(ng+2)]
	se4 <- sqrt(diag(bb4$cov))[(ng+1):(ng+2)]

# 3.3. Wald tests
	R = c(rep(0,ng),1,0)
	if(est.opt==1){theta = a1$coef}
	if(est.opt==2){theta = c(fit.sqr$par[1:ng],beta.bsqr)}
	if(est.opt==3){theta = coef.jack}	
	wald1 <- (t(R) %*% theta) %*% solve(t(R) %*% bb2$covs %*% R) %*% (t(R) %*% theta)
	wald2 <- (t(R) %*% theta) %*% solve(t(R) %*% bb2$cov %*% R) %*% (t(R) %*% theta)
	wald3 <- (t(R) %*% theta) %*% solve(t(R) %*% bb3$cov %*% R) %*% (t(R) %*% theta)

# 3.4. Score test
	a2 <- rq(y~factor(state)+x1-1,tau=tt,data=data.p)
	uhat2  <- a2$resid
	x.ext2 <- cbind(x[,3:(ng+2)],x1)
	dx2 = dx-1
	
	# FE-SQR
	if(est.opt %in% c(2,3)){
		s1 <- sd(uhat2)
		hn <- const1*s1*(ng*nt)^(-1/7)
		fit.sqr <- optim(a2$coef,rq.opt2,tau=tt,hh=hn,y=y,x=x1,D=D1,n=ng,t=nt)
		beta.sqr <- fit.sqr$par[(ng+1):(ng+dx2)]
	}
	# Analytical bias corrected estimator
	if(est.opt==2){
		ehat.f <- y-D1%*%fit.sqr$par[1:ng]-x1*fit.sqr$par[(ng+1):(ng+dx2)]
		s2 <- sd(ehat.f)
		bn <- const2*s2*nt^(-1/5)
		beta.bsqr <- rq.bsqr(ehat.f,bn,x1,beta.sqr,ng,nt,tt)
		uhat2 <- y-D1%*%fit.sqr$par[1:ng]-x1*beta.bsqr
		uhat2 <- as.vector(uhat2)	
	}
	# Jackknife
	if(est.opt==3){	
		fit.sqr.S1 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S1==1],x=x1[ind.S1==1],D=D.S,n=ng,t=nt.s)
		fit.sqr.S2 <- optim(fit.sqr$par,rq.opt2,tau=tt,hh=hn,y=y[ind.S2==1],x=x1[ind.S2==1],D=D.S,n=ng,t=nt.s)
		beta.sqr.S1 <- fit.sqr.S1$par
		beta.sqr.S2 <- fit.sqr.S2$par
		beta.jack <- 2*beta.sqr-(beta.sqr.S1+beta.sqr.S2)[(ng+1):(ng+dx2)]/2
		coef.jack <- 2*fit.sqr$par - (beta.sqr.S1+beta.sqr.S2)/2
		uhat2 <- y-D1%*%coef.jack[1:ng]-x1*coef.jack[(ng+1):(ng+dx2)]
		uhat2 <- as.vector(uhat2)
	}

	# save outcomes
	sc  <- Sco.test(treat,x.ext2,uhat2,tau=tt,nt,ng,mt2,bias=0,ker=1)
	sc2 <- Sco.test(treat,x.ext2,uhat2,tau=tt,nt,ng,mt2,bias=1,ker=1)
	sco1 <- sc$score
	sco2 <- sc$score.r
	sco3 <- sc2$score.r
	
# 4. Collect outcomes
	waldstat <- c(waldstat,wald1)
	waldstat2 <- c(waldstat2,wald2)
	waldstat3 <- c(waldstat3,wald3)
	scostat <- c(scostat,sco1)	
	scostat2 <- c(scostat2,sco2)
	scostat3 <- c(scostat3,sco3)
	coef.rq <- c(coef.rq,theta[ng+1])
	
r.ratio <- c((sum(abs(tstat.rq1)>1.96)/j),(sum(abs(tstat.rq2)>1.96)/j),(sum(abs(tstat.ols)>1.96)/j))	# rejection ratios from the conventioal methods 

r.ratio2 <- c((sum(scostat>3.84)/j),(sum(scostat2>3.84)/j),(sum(scostat3>3.84)/j),(sum(waldstat>3.84)/j),(sum(waldstat2>3.84)/j),(sum(waldstat3>3.84)/j))	# rejection ratios from the CCM estimators
print("the number of simulation is"); print(j)
}# end of the loop

save(scostat,scostat2,waldstat,waldstat2,waldstat3,tstat.rq1,tstat.rq2,tstat.ols,r.ratio,r.ratio2,coef.rq,LL,tt,rho.st,est.opt,file=paste("unemp_sim","_tau_",tt,"_est_",est.opt,".rda",sep=""))
}
